@@ -8,7 +8,7 @@ package IO::Async::Function;
 use strict;
 use warnings;
 
-our $VERSION = '0.38';
+our $VERSION = '0.39';
 
 use base qw( IO::Async::Notifier );
 use IO::Async::Timer::Countdown;
@@ -406,7 +406,7 @@ sub _new_worker
       ( map { $_ => $self->{$_} } qw( code setup exit_on_die ) ),
 
       on_finish => $self->_capture_weakself( sub {
-         my $self = shift;
+         my $self = shift or return;
          my ( $worker ) = @_;
 
          if( @{ $worker->{on_result_queue} } ) {
