@@ -8,7 +8,7 @@ package IO::Async::Notifier;
 use strict;
 use warnings;
 
-our $VERSION = '0.38';
+our $VERSION = '0.39';
 
 use Carp;
 use Scalar::Util qw( weaken );
@@ -129,6 +129,17 @@ details on the parameters that exist, and their uses. Some parameters may only
 support being set once at construction time, or only support being changed if
 the object is in a particular state.
 
+The following parameters are supported by all Notifiers:
+
+=over 8
+
+=item notifier_name => STRING
+
+Optional string used to identify this particular Notifier. This value will be
+returned by the C<notifier_name> method.
+
+=back
+
 =cut
 
 =head1 CONSTRUCTOR
@@ -172,6 +183,10 @@ sub new
    return $self;
 }
 
+=head1 METHODS
+
+=cut
+
 =head2 $notifier->configure( %params )
 
 Adjust the named parameters of the C<Notifier> as given by the C<%params>
@@ -185,6 +200,10 @@ sub configure
    my $self = shift;
    my %params = @_;
 
+   foreach (qw( notifier_name )) {
+      $self->{$_} = delete $params{$_} if exists $params{$_};
+   }
+
    # We don't recognise any configure keys at this level
    if( keys %params ) {
       my $class = ref $self;
@@ -222,6 +241,20 @@ sub __set_loop
    $self->_add_to_loop( $self->{loop} ) if $self->{loop};
 }
 
+=head2 $name = $notifier->notifier_name
+
+Returns the name to identify this Notifier. If a has not been set, it will
+return the empty string. Subclasses may wish to override this behaviour to
+return some more useful information, perhaps from configured parameters.
+
+=cut
+
+sub notifier_name
+{
+   my $self = shift;
+   return $self->{notifier_name} || "";
+}
+
 =head1 CHILD NOTIFIERS
 
 During the execution of a program, it may be the case that certain IO handles
@@ -426,6 +459,18 @@ the CODE ref internally for efficiency.
 The C<$code> argument may also be a plain string, which will be used as a
 method name; the returned CODE ref will then invoke that method on the object.
 
+If the C<$mref> CODE reference is being stored in some object other than the
+one it refers to, remember that since the Notifier is only weakly captured, it
+is possible that it has been destroyed by the time the code runs, and so the
+reference will be passed as C<undef>. This should be protected against by the
+code body.
+
+ $other_object->{on_event} = $notifier->_capture_weakself( sub {
+    my $notifier = shift or return;
+    my ( @event_args ) = @_;
+    ...
+ } );
+
 =cut
 
 sub _capture_weakself
@@ -477,6 +522,9 @@ function's arguments.
 The C<$code> argument may also be a plain string, which will be used as a
 method name; the returned CODE ref will then invoke that method on the object.
 
+As with C<_capture_weakself>, care should be taken against Notifier
+destruction if the C<$mref> CODE reference is stored in some other object.
+
 =cut
 
 sub _replace_weakself
